"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = exports.processDeploymentGroup = exports.processDeploymentTarget = void 0;
const core_1 = require("@takomo/core");
const util_1 = require("@takomo/util");
const child_process_1 = require("child_process");
const cockatiel_1 = require("cockatiel");
const ramda_1 = __importDefault(require("ramda"));
const runChildProcess = async (command, cwd, env, inputs) => new Promise((resolve, reject) => {
    const child = child_process_1.exec(command, { cwd, env }, (error, stdout, stderr) => {
        if (error) {
            reject(error);
        }
        else {
            resolve(stdout);
        }
    });
    inputs.forEach((r) => {
        child.stdin?.write(`${r}`);
    });
    child.stdin?.end();
});
const getDeploymentRole = (currentIdentity, group, target, roleName) => {
    const deploymentRole = target.deploymentRole ?? group.deploymentRole;
    if (deploymentRole) {
        return deploymentRole;
    }
    const deploymentRoleName = roleName ?? target.deploymentRoleName ?? group.deploymentRoleName;
    if (!deploymentRoleName) {
        return undefined;
    }
    const deploymentAccountId = target.accountId ?? currentIdentity.accountId;
    return {
        iamRoleArn: `arn:aws:iam::${deploymentAccountId}:role/${deploymentRoleName}`,
    };
};
const captureValue = ({ captureAfterLine, captureLastLine, captureBeforeLine, }, output) => {
    const lines = output.trim().split("\n");
    if (captureLastLine) {
        return (ramda_1.default.last(lines) ?? "") + "\n";
    }
    if (!captureAfterLine && !captureBeforeLine) {
        return output;
    }
    const start = captureAfterLine
        ? lines.findIndex((item) => item === captureAfterLine)
        : -1;
    const end = captureBeforeLine
        ? lines.findIndex((item) => item === captureBeforeLine)
        : -1;
    const endLineNumber = end === -1 ? lines.length : end;
    return lines
        .slice(start + 1, endLineNumber)
        .map((l) => `${l}\n`)
        .join("");
};
const runJsMapFunction = async ({ ctx, mapCommand, group, target, logger, credentials, }) => {
    const mapFunctionFullPath = util_1.expandFilePath(ctx.projectDir, mapCommand);
    logger.debug(`Run map function from file: ${mapFunctionFullPath}`);
    // eslint-disable-next-line
    const mapperFn = require(mapFunctionFullPath);
    return mapperFn({
        credentials,
        target: target,
        deploymentGroupPath: group.path,
    });
};
const runMapProcessCommand = async ({ ctx, mapCommand, group, target, logger, credentials, input, }) => {
    logger.debug(`Run map command: ${mapCommand}`);
    const targetJson = JSON.stringify(target);
    const env = {
        ...util_1.deepCopy(process.env),
        TKM_TARGET_NAME: target.name,
        TKM_TARGET_JSON: targetJson,
        TKM_DEPLOYMENT_GROUP_PATH: group.path,
        AWS_ACCESS_KEY_ID: credentials.accessKeyId,
        AWS_SECRET_ACCESS_KEY: credentials.secretAccessKey,
        AWS_SESSION_TOKEN: credentials.sessionToken,
        AWS_SECURITY_TOKEN: credentials.sessionToken,
    };
    const output = await runChildProcess(mapCommand, ctx.projectDir, env, [
        targetJson,
    ]);
    return captureValue(input, output);
};
const runMapCommand = (props) => props.mapCommand.startsWith("js:")
    ? runJsMapFunction({ ...props, mapCommand: props.mapCommand.substr(3) })
    : runMapProcessCommand(props);
const runReduceProcessCommand = ({ credentials, ctx, reduceCommand, targetResults, }) => {
    const env = {
        ...util_1.deepCopy(process.env),
        AWS_ACCESS_KEY_ID: credentials.accessKeyId,
        AWS_SECRET_ACCESS_KEY: credentials.secretAccessKey,
        AWS_SESSION_TOKEN: credentials.sessionToken,
        AWS_SECURITY_TOKEN: credentials.sessionToken,
    };
    return runChildProcess(reduceCommand, ctx.projectDir, env, targetResults);
};
const runJsReduceFunction = ({ credentials, ctx, reduceCommand, targetResults, }) => {
    const fullReduceFunctionPath = util_1.expandFilePath(ctx.projectDir, reduceCommand);
    // eslint-disable-next-line
    const reduceFn = require(fullReduceFunctionPath);
    return reduceFn({
        credentials,
        targets: targetResults,
    });
};
const runReduceCommand = async (props) => props.reduceCommand.startsWith("js:")
    ? runJsReduceFunction({
        ...props,
        reduceCommand: props.reduceCommand.substr(3),
    })
    : runReduceProcessCommand(props);
const getCredentialManager = async (ctx, deploymentRole) => deploymentRole
    ? await ctx.credentialManager.createCredentialManagerForRole(deploymentRole.iamRoleArn)
    : ctx.credentialManager;
const processDeploymentTarget = async (props, group, target, timer, state, logger) => {
    const { ctx, input } = props;
    const { mapCommand, mapRoleName } = input;
    if (state.failed) {
        timer.stop();
        return {
            message: "Cancelled",
            status: "CANCELLED",
            success: false,
            name: target.name,
            timer,
        };
    }
    logger.info("Start run");
    try {
        const currentIdentity = await ctx.credentialManager.getCallerIdentity();
        const deploymentRole = getDeploymentRole(currentIdentity, group, target, mapRoleName);
        const credentialManager = await getCredentialManager(ctx, deploymentRole);
        if (target.accountId) {
            const identity = await credentialManager.getCallerIdentity();
            if (identity.accountId !== target.accountId) {
                throw new util_1.TakomoError(`Current credentials belong to AWS account ${identity.accountId}, but the deployment target can be run only against account: ${target.accountId}`);
            }
        }
        const credentials = await credentialManager.getCredentials();
        const value = await runMapCommand({
            mapCommand,
            ctx,
            logger,
            target,
            group,
            credentials,
            input,
        });
        timer.stop();
        return {
            status: "SUCCESS",
            name: target.name,
            success: true,
            message: "Success",
            value,
            timer,
        };
    }
    catch (error) {
        timer.stop();
        logger.error(`Map command failed for target '${target.name}'`, error);
        return {
            status: "FAILED",
            name: target.name,
            success: false,
            message: "Error",
            error,
            timer,
        };
    }
};
exports.processDeploymentTarget = processDeploymentTarget;
const convertToOperation = (props, group, timer, state, target, results, policy, listener) => () => policy.execute(async () => {
    await listener.onTargetBegin();
    const result = await exports.processDeploymentTarget(props, group, target, timer.startChild(target.name), state, props.io.childLogger(target.name));
    results.push(result);
    await listener.onTargetComplete();
    return result;
});
/**
 * @hidden
 */
const processDeploymentGroup = async (props, group, timer, state, listener) => {
    const { io, input } = props;
    const { concurrentTargets } = input;
    io.info(`Run deployment group: ${group.path}`);
    const results = new Array();
    const deploymentPolicy = cockatiel_1.Policy.bulkhead(concurrentTargets, 10000);
    const operations = group.targets.map((target) => convertToOperation(props, group, timer, state, target, results, deploymentPolicy, listener));
    await Promise.all(operations.map((o) => o()));
    timer.stop();
    return {
        ...core_1.resolveCommandOutputBase(results),
        results,
        path: group.path,
        timer,
    };
};
exports.processDeploymentGroup = processDeploymentGroup;
const run = async (props) => {
    const { input, plan, listener, ctx, io } = props;
    const childTimer = input.timer.startChild("run");
    const results = new Array();
    const state = { failed: false };
    for (const group of plan.groups) {
        const result = await exports.processDeploymentGroup(props, group, childTimer.startChild(group.name), state, listener);
        results.push(result);
    }
    const outputBase = core_1.resolveCommandOutputBase(results);
    const targetResults = results
        .map((r) => r.results)
        .flat()
        .map((r) => r.value);
    if (!outputBase.success || !input.reduceCommand) {
        childTimer.stop();
        return {
            ...outputBase,
            result: targetResults,
            timer: childTimer,
            outputFormat: input.outputFormat,
        };
    }
    const reduceCredentialManager = input.reduceRoleArn
        ? await ctx.credentialManager.createCredentialManagerForRole(input.reduceRoleArn)
        : ctx.credentialManager;
    const reduceCredentials = await reduceCredentialManager.getCredentials();
    try {
        const result = await runReduceCommand({
            targetResults,
            ctx,
            credentials: reduceCredentials,
            reduceCommand: input.reduceCommand,
            logger: io,
        });
        childTimer.stop();
        return {
            ...outputBase,
            result,
            timer: childTimer,
            outputFormat: input.outputFormat,
        };
    }
    catch (error) {
        childTimer.stop();
        io.error(`Reduce command failed`, error);
        return {
            error,
            result: undefined,
            message: "Reduce command failed",
            success: false,
            status: "FAILED",
            timer: childTimer,
            outputFormat: input.outputFormat,
        };
    }
};
exports.run = run;
//# sourceMappingURL=run.js.map